/**
 * \file 
 * \author lcs09
 * \brief Test suite per le funzioni sugli intervalli
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "CUnit.h"
#include "TestDB.h"
#include "Basic.h"
#include "intervals.h"
#include "ilist.h"

/** permessi ben formati */
char* permessiok[] = {
  "CN445JK 12/12/2008-13:30 13/12/2008-14:30",
  "DD115PU 01/01/2009-00:00 30/06/2009-00:00",
  "CN445JK 10/02/2009-13:30 11/02/2009-14:30",
  "DD115PU 01/01/2008-00:00 30/08/2008-23:10",
  "AS105SU 13/01/2008-00:00 30/09/2008-20:15",
  "BU995PK 01/03/2009-10:00 01/03/2009-20:30",
  "KK876VB 01/02/2009-00:00 28/02/2009-22:00",
  "KK876VB 01/04/2009-00:00 30/04/2009-22:00",
  "DS009LU 01/01/2009-00:00 30/08/2009-23:10",
  "KC044ZZ 21/01/2009-09:00 22/01/2009-22:00",
  "AU663XZ 21/01/2009-09:00 22/01/2009-22:00",
  NULL
};

/** permessi mal formati */
char* permessibad[] = {
  "AA005PP 11/11/2009-15:30 13/02/2009-19:30", /* date non tornano */
  "CN44K 12/12/2008-13:30 13/12/2008-14:30", /* corto */
  "CN4456K 12/12/2008-13:30 13/12/2008-14:30", /* alfanumerico */
  "CN445JK 12/15/2008-13:30 13/12/2008-14:30", /* data sbagliata */
  "CN445JK 12/12/b008-13:30 13/12/2008-14:30", /* anno sbagliato */
  "CN445JK 12/12/2008-14:30 12/12/2008-13:30", /* intervallo vuoto */
  NULL
};

/** Funzione di inizializzazione della suite */
int success_init(void) { return 0; }
/** Funzione di clean up della suite */
int success_clean(void) { return 0; }

/**
 * Questa funzione converte alcune strunghe in struct permessi e viceversa testando le funzioni di conversione convertiRecord e convertiPerm
 */
void run_testIntervalli (void) {
  int i,a;
  permesso_t* pr=NULL;
  char r[LRECORD+1];
  

  /* 
     test di conversione da stringa a a struttura permesso
  */
  for(i=0;permessiok[i]!=NULL;i++) {
    
    /*
      conversione da record a struttura permesso
    */
    pr = convertiRecord(permessiok[i]);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pr); 
    
    /*
      stampa l'intero permesso
    */
    stampaPermesso(pr);

    /*
      conversione inversa (struct -- record)
    */
    a = convertiPermesso(pr,r);
    CU_ASSERT_EQUAL_FATAL(a,0);

    /*
      abbiamo ottenuto la stessa cosa?
    */

    a = strcmp(permessiok[i],r);
    CU_ASSERT_EQUAL_FATAL(a,0);

    /* 
       liberiamo la memoria
     */
    free(pr);
    pr = NULL;
  }

  /* 
     test conversione 
     stringhe mal formate 
  */
  for(i=0;permessibad[i]!=NULL;i++) {
    /* 
       riconosco il problema ?
     */
    pr = convertiRecord(permessibad[i]);
    CU_ASSERT_PTR_NULL_FATAL(pr); 
  }
}

/**
 * Questa funzione testa le funzioni in ilist.h
 */
void run_testListeIntervalli (void) {
  int i,a;
  /*  char r[LRECORD+1];*/
  permesso_t* pr=NULL;
  elem_t* lista=NULL;
  elem_t *inter=NULL;



  for(i=0;permessiok[i]!=NULL;i++) {
    /* 
       converto il permesso 
    */
    pr = convertiRecord(permessiok[i]);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pr); 

    /* 
       estraggo l'intervallo e lo aggiungo alla lista
    */
    a = addIntervallo(&lista,&pr->in);
    CU_ASSERT_EQUAL_FATAL(a,0);

    /*
      stampa l'intervallo
    */
    stampaIntervallo(&pr->in);

    free(pr);
    pr=NULL;
  }

  for(i=0;permessiok[i]!=NULL;i++) {
    /* 
       converto il permesso 
    */
    pr = convertiRecord(permessiok[i]);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pr); 

    /* 
       c'e' nella lista ?
       (alcuni intervalli sono generati piu' volte ma compaiono 
        in singola copia nella lista)
    */
    
    inter = NULL;
    inter = findIntervallo(lista,&pr->in);
    if ( inter != NULL ) {
      CU_ASSERT_EQUAL_FATAL(inter->in.inizio,pr->in.inizio);
      CU_ASSERT_EQUAL_FATAL(inter->in.fine,pr->in.fine);
    }
    
    /* 
       lo rimuovo e vedo se c'e' ancora
    */
    
    a = removeIntervallo(&lista,&pr->in);
    CU_ASSERT_EQUAL_FATAL(a,0);
    
    inter = findIntervallo(lista,&pr->in);
    CU_ASSERT_EQUAL_FATAL(inter,NULL);

    free(pr);
    pr = NULL;
  }

  /* creo e dealloco una lista di intervalli */
  for(i=0;permessiok[i]!=NULL;i++) {
    /* 
       converto il permesso e lo inserisco
    */
    pr = convertiRecord(permessiok[i]);
    a = addIntervallo(&lista,&pr->in);
    CU_ASSERT_EQUAL_FATAL(a,0);
    
    free(pr);
    pr = NULL;
  }

  /*
    dealloco la lista e termino 
  */

    deallocaListIntervallo(lista);

}
/**
 * \test
 *
 * Testa le funzioni di conversione di permessi/intervalli
 */
void interval_test (void)
{
    run_testIntervalli();
}

/**
 * \test
 *
 * Testa le funzioni sulle liste di intervalli
 */
void listinterval_test (void)
{
    run_testListeIntervalli();
}

/** 
 * Definizione della suite. 
 */
void add_suite_testinterval(void)
{
    CU_pSuite interval_suite = NULL;

    interval_suite = CU_add_suite("Test Intervalli", success_init, success_clean);
    CU_add_test(interval_suite, "Test 1 conversioni di intervalli", interval_test);
    CU_add_test(interval_suite, "Test 2 liste di intervalli", listinterval_test);

}

